<?php

/**
 * Photo Spot
 *
 * @version 1.0
 * @author Creative Pulse
 * @copyright Creative Pulse 2009
 * @license GNU/GPLv2
 * @link http://www.creativepulse.eu
 */


class PhotoSpotDatasourceDirectoryDisplay extends PhotoSpotDatasourceDisplay {

	function init($viewer) {
		$utils =& PhotoSpotUtils::get_instance();

		$path = trim($this->params->get('img_dir', ''));
		if (empty($path))
			return JText::_('No path given');

		$type = $utils->norm_file_paths($path, false, $this->img_url_path, $this->img_abs_path);
		if ($this->img_url_path != 'images' && substr($this->img_url_path, 0, 7) != 'images/')
			return JText::sprintf('Path &quot;%s&quot; is not in the images directory', htmlspecialchars($path));
		if (strpos($path, '..') !== false)
			return JText::sprintf('Path &quot;%s&quot; contains parent directory symbol (..)', htmlspecialchars($path));
		if ($type == 'notfound')
			return JText::sprintf('Path &quot;%s&quot; does not exist', htmlspecialchars($path));
		if ($type != 'directory')
			return JText::sprintf('Path &quot;%s&quot; is not a valid directory', htmlspecialchars($path));

		$path = trim($this->params->get('thumb_dir', ''));
		if ($path != '') {
			$type = $utils->norm_file_paths($path, false, $this->thumb_url_path, $this->thumb_abs_path);
			if (strpos($path, '..') !== false)
				return JText::sprintf('Path &quot;%s&quot; contains parent directory symbol (..)', htmlspecialchars($path));
			if ($type == 'notfound')
				return JText::sprintf('Path &quot;%s&quot; does not exist', htmlspecialchars($path));
			if ($type != 'directory')
				return JText::sprintf('Path &quot;%s&quot; is not a valid directory', htmlspecialchars($path));
		}

		$order = $this->params->get('order', '');

		$filters_src = trim($this->params->get('filter', ''));
		if ($filters_src == '')
			$filters_src = '*.jpg|*.gif|*.png|*.jpeg';

		$filters = array();
		foreach (explode('|', $filters_src) as $filter) {
			$pattern = '';
			for ($i = 0, $len = strlen($filter); $i < $len; $i++)
				if (strpos('abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789', $filter{$i}) !== false)
					$pattern .= $filter{$i};
				else if ($filter{$i} == '*')
					$pattern .= '.*';
				else
					$pattern .= '\\' . $filter{$i};

			if ($pattern != '')
				$filters[] = '/^' . $pattern . '$/i';
		}

		if ($dp = opendir($this->img_abs_path)) {
			while (false !== ($file = readdir($dp))) {
				if ($file == '.' || $file == '..')
					continue;

				$ok = false;
				foreach ($filters as $filter)
					if (preg_match($filter, $file)) {
						$ok = true;
						break;
					}

				if ($ok) {
					$item = new PhotoSpotDisplayItem();

					$item->img_url = $this->img_url_path . $file;
					$item->img_abs = $this->img_abs_path . $file;

					if (!empty($this->thumb_url_path) && file_exists($this->thumb_abs_path . $file)) {
						$item->thumb_url = $this->thumb_url_path . $file;
						$item->thumb_abs = $this->thumb_abs_path . $file;
					}

					$item->mtime = filemtime($item->img_abs);

					$item->ext = strrchr($file, '.');
					if (!empty($item->ext)) {
						$item->ext = strtolower(substr($item->ext, 1));

						$text_filename = $this->img_abs_path . substr($file, 0, -strlen($item->ext)) . 'txt';
						if (file_exists($text_filename) && $fp = @fopen($text_filename, 'r')) {
							$item->title = trim(fread($fp, filesize($text_filename)));
							fclose($fp);
						}
					}

					$info = getimagesize($item->img_abs);
					$item->img_width = $info[0];
					$item->img_height = $info[1];
					$item->mime = @$info['mime'];

					if ($item->thumb_abs !== false) {
						$info = getimagesize($item->thumb_abs);
						$item->thumb_width = $info[0];
						$item->thumb_height = $info[1];
					}

					$this->items[] = $item;
				}
			}
			closedir($dp);

			// count items
			$this->items_count = count($this->items);

			// sort
			switch ($order) {
				case 'filename':
					usort($this->items, array('PhotoSpotDatasourceDirectoryDisplay', 'sort_filename'));
					break;

				case 'title':
					usort($this->items, array('PhotoSpotDatasourceDirectoryDisplay', 'sort_title'));
					break;

				case 'mdate_desc':
					usort($this->items, array('PhotoSpotDatasourceDirectoryDisplay', 'sort_mdate_desc'));
					break;

				case 'mdate_asc':
					usort($this->items, array('PhotoSpotDatasourceDirectoryDisplay', 'sort_mdate_asc'));
					break;

				case 'random':
					$count = $viewer->limit;
					if ($count == 0 || $count > $this->items_count)
						$count = $this->items_count;

					$new_list = array();
					for ($i = 0; $i < $count; $i++)
						$new_list = array_merge($new_list, array_splice($this->items, mt_rand(0, count($this->items) - 1), 1));

					$this->items = $new_list;
					$this->items_count = count($this->items);
					break;

			}
		}

		if ($order != 'random' && $viewer->limit > 0 && $this->items_count > $viewer->limit)
			$this->items = array_slice($this->items, $viewer->limitstart, $viewer->limit);

		return '';
	}

	function sort_filename($a, $b) {
		$al = strtolower(basename($a->img_url));
		$bl = strtolower(basename($b->img_url));
		if ($al == $bl)
			return 0;

		return $al > $bl ? 1 : -1;
	}

	function sort_title($a, $b) {
		$al = strtolower($a->get_title());
		$bl = strtolower($b->get_title());
		if ($al == $bl)
			return 0;

		return $al > $bl ? 1 : -1;
	}

	function sort_mdate_desc($a, $b) {
		if ($a->mtime == $b->mtime) {
			$al = strtolower($a->get_title());
			$bl = strtolower($b->get_title());
			if ($al == $bl)
				return 0;

			return $al > $bl ? 1 : -1;
		}

		return $a->mtime < $b->mtime ? 1 : -1;
	}

	function sort_mdate_asc($a, $b) {
		if ($a->mtime == $b->mtime) {
			$al = strtolower($a->get_title());
			$bl = strtolower($b->get_title());
			if ($al == $bl)
				return 0;

			return $al > $bl ? 1 : -1;
		}

		return $a->mtime > $b->mtime ? 1 : -1;
	}

}

$engine = new PhotoSpotDatasourceDirectoryDisplay();

?>